#include "libch57x.h"

#define LED_PORT     LIBCH57X_GPIO_BANK_B
#define LED_PIN      0U
#define LED_PWM_CH   2U
#define LED_PWM_MASK (1U << LED_PWM_CH)

static volatile uint32_t s_tick_count = 0UL;

static void tick_delay(uint32_t msec) {
    uint32_t tick_start = s_tick_count;

    while (s_tick_count - tick_start <= msec) {
        /* -- */
    }
}

int main(int argc, const char *argv[]) {
    SystemCoreClockUpdate();

    SysTick_Config(SystemCoreClock / 1000); /* 1kHz */

    libch57x_gpio_config_t led_cfg = {
        .bank     = LED_PORT,
        .pin_mask = (1U << LED_PIN),
        .mode     = LIBCH57X_GPIO_MODE_OUTPUT_5MA,
    };

    libch57x_gpio_config(&led_cfg);

    libch57x_pwm_config_t pwm_cfg = {
        .cyc_mode = LIBCH57x_PWM_CYC_MODE_8B,
        .cyc_sel  = LIBCH57x_PWM_CYCLE_SEL_NORM,
        .stag_en  = 0U,
    };

    libch57x_pwm_config(PWM, &pwm_cfg);
    libch57x_pwm_clock_div(PWM, 32);

    libch57x_pwm_polarity(PWM, LED_PWM_MASK, true);
    libch57x_pwm_duty(PWM, LED_PWM_MASK, 4);

    libch57x_pwm_enable(PWM, LED_PWM_MASK, true);

    uint8_t new_duty = 0;
    uint8_t duty_dir = 0;

    for (;;) {
        tick_delay(10);
        if (duty_dir == 0U) {
            libch57x_pwm_duty(PWM, LED_PWM_MASK, new_duty++);
            if (new_duty == 0) {
                duty_dir = 1U;
            }
        } else {
            libch57x_pwm_duty(PWM, LED_PWM_MASK, --new_duty);
            if (new_duty == 0) {
                duty_dir = 0U;
            }
        }
    }
}

void SysTick_Handler(void) {
    s_tick_count++;
}